from __future__ import division
import matplotlib
import numpy as np
from scipy import special
from sympy import mpmath
import matplotlib.cm as cm
import matplotlib.mlab as mlab 
import matplotlib.pyplot as plt
import warnings

mu_0 = 1.25663706e-6
M=1./mu_0                     # magnetization of neodymium
h=1e-2                        # Height of magnet (which is a cylinder)
I = M*h
r_circle=5e-2
V_kule = 1e-9


# Returns a function taking (r, z) as parameters and returns (B_r,
# B_z) at that point from a current ring with current i and radius a
# at origo.
def B_of_ring(i, a):
    # See http://www.netdenizen.com/emagnettest/offaxis/?offaxisloop
    def retval(r, z):
        x=z
        r = np.abs(r)
        alpha = r/a
        beta = x/a
        gamma = x/r
        # B_r is 0 on axis (where r==0), and B_r is multiplied by gamma
        gamma[r==0] = 0
        Q = (1+alpha)**2 + beta**2
        k = np.sqrt(4*alpha/Q)
        B_0 = i*mu_0/(2*a)
        E = lambda kp: special.ellipe(kp**2)
        K = lambda kp: special.ellipk(kp**2)
        myk = B_0/(np.pi*np.sqrt(Q))
        B_x = myk*(E(k)*(1-alpha**2-beta**2)/(Q-4*alpha)+K(k))
        B_r = myk*gamma*(E(k)*(1+alpha**2+beta**2)/(Q-4*alpha)-K(k))
        return (B_r, B_x)
    return retval


# Returns the B field of a cylinder with surface current K, radius a
# and height h. The cylinder is modelled as n rings centered at r=0
# and delta Z=h/n. Origo of the cylinder is at the center of the
# top-most ring.
def B_of_cylinder(K, a, h, n=20):
    delta = h/float(n)
    i = K*delta
    B_function = B_of_ring(i, a)

    def retval(r, z):
        B = [np.zeros(r.shape), np.zeros(z.shape)]
        for m in xrange(n):
            z0 = -delta*(float(m)+0.5)
            g = B_function(r, z-z0)
            B[0] += g[0]
            B[1] += g[1]
        return B

    return retval

def BuildForceField(magnet_list, r, z):
    B = [np.zeros(r.shape),np.zeros(r.shape)]
    for m in magnet_list:
        h = m.height
        r_c = m.radius
        sgn = m.sign
        z_0 = m.z_0
        b_r,b_z = B_of_cylinder(sgn*M, r_c, h, m.gran)(r, z-z_0)
        B[0] += b_r
        B[1] += b_z

    B_squared = B[0]**2+B[1]**2

    f = np.gradient(3*V_kule/mu_0*B_squared, z[1][0]-z[0][0], r[0][1]-r[0][0])
    f_mag = np.sqrt(f[0]**2+f[1]**2)
    f_mag[np.isinf(f_mag)] = np.nan
    return f_mag

# Returns the B field of a cylindersymetric magnet with magnetization
# M, walls following the curve of f, with boundaries on the z-axis at
# minz and maxz. The cylinder will be modelled by n rings.
def B_of_function(M, f, minz, maxz, n=20):
    delta = (maxz-minz)/float(n)
    print delta, maxz, minz
    i = M*delta
    
    def retval(r, z):
        B = [np.zeros(r.shape), np.zeros(z.shape)]
        for m in xrange(n):
            z0 = delta*float(m)+minz
            print z0, f(z0)
            g = B_of_ring(i, f(z0))(r, z-z0)
            B[0] += g[0]
            B[1] += g[1]
        return B

    return retval

def BuildFunctionForceField(magnet_list, r, z):
    B = [np.zeros(r.shape),np.zeros(r.shape)]
    for m in magnet_list:
        minz = m.minz
        maxz = m.maxz
        f = m.rFunction
        sgn = m.sign
        z_0 = m.z_0
        b_r,b_z = B_of_function(sgn*M, f, minz, maxz, m.gran)(r, z-z_0)
        B[0] += b_r
        B[1] += b_z

    B_squared = B[0]**2+B[1]**2

    f = np.gradient(3*V_kule/mu_0*B_squared, z[1][0]-z[0][0], r[0][1]-r[0][0])
    f_mag = np.sqrt(f[0]**2+f[1]**2)
    f_mag[np.isinf(f_mag)] = np.nan
    return f_mag

def ironDustSphere(d, ironDensity, r, z):
    """
    Create a sphere of iron dust of diameter d centered at r,z.

    Each dust particle has dimension V_kule

    ironDensity is the density of the iron, given in mm3/mm3.

    Returns an dxd array, each point giving the number of iron dust particles at (i-d/2,j-d/2)+(r,z)
    """
    pass
    

    
